<?php

namespace FiloBlu\DynamicCustomOptions\Model;

use Magento\Rule\Model\AbstractModel;

/**
 * Class Rule
 * @package FiloBlu\DynamicCustomOptions\Model
 *
 * @method int|null getRuleId()
 * @method Rule setRuleId(int $id)
 */
class DynamicCustomOptions extends AbstractModel {

    /**
     * Prefix of model events names
     *
     * @var string
     */
    protected $_eventPrefix = 'filoblu_dynamiccustomoptions';

    /**
     * Parameter name in event
     *
     * In observe method you can use $observer->getEvent()->getRule() in this case
     *
     * @var string
     */
    protected $_eventObject = 'dynamiccustomoptions';

    /** @var \Magento\SalesRule\Model\Rule\Condition\CombineFactory */
    protected $condCombineFactory;

    /** @var \Magento\SalesRule\Model\Rule\Condition\Product\CombineFactory */
    protected $condProdCombineF;
    protected $productCollectionFactory;
    protected $sqlBuilder;
    protected $productOptionFactory;

    public function __construct(
    \Magento\Framework\Model\Context $context, \Magento\Framework\Registry $registry, \Magento\Framework\Data\FormFactory $formFactory, \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate, \Magento\CatalogRule\Model\Rule\Condition\CombineFactory $condCombineFactory, \Magento\CatalogRule\Model\Rule\Condition\ProductFactory $condProdCombineF, \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollectionFactory, \Magento\Rule\Model\Condition\Sql\Builder $sqlBuilder, \Magento\Catalog\Model\Product\OptionFactory $productOptionFactory, \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null, \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null, array $data = []
    ) {
        $this->condCombineFactory = $condCombineFactory;
        $this->condProdCombineF = $condProdCombineF;
        $this->productCollectionFactory = $productCollectionFactory;
        $this->sqlBuilder = $sqlBuilder;
        $this->productOptionFactory = $productOptionFactory;
        parent::__construct($context, $registry, $formFactory, $localeDate, $resource, $resourceCollection, $data);
    }

    /**
     * Set resource model and Id field name
     *
     * @return void
     */
    protected function _construct() {
        parent::_construct();
        $this->_init('FiloBlu\DynamicCustomOptions\Model\ResourceModel\DynamicCustomOptions');
        $this->setIdFieldName('dynamiccustomoptions_id');
    }

    /**
     * Get rule condition combine model instance
     *
     * @return \Magento\CatalogRule\Model\Rule\Condition\Combine
     */
    public function getConditionsInstance() {
        return $this->condCombineFactory->create();
    }

    /**
     * Get rule condition product combine model instance
     *
     * @return \Magento\CatalogRule\Model\Rule\Condition\Product
     */
    public function getActionsInstance() {
        return $this->condProdCombineF->create();
    }

    public function beforeSave() {

//        if (is_array($this->getData('store_ids'))) {
//            $this->setData('store_ids', implode(',', $this->getData('store_ids')));
//        }

        return parent::beforeSave();
    }

    public function afterSave() {

        $this->doActions(false);

        return parent::afterSave();
    }

    public function beforeDelete() {

        $customOptionsIdsToDelete = $this->getResource()->loadCustomOptionsToDelete($this->getId());
        $this->deleteCustomOptions($customOptionsIdsToDelete);

        return parent::beforeDelete();
    }

    public function doActions($executedByCron) {
        if ($executedByCron && !$this->getIsCronActive()) {
            return;
        }
        
        if (!$executedByCron && !$this->getDoActionsOnSave()) {
            return;
        }

        $options = json_decode($this->getJsonConfig(), true);

        // If Rule is not active, delete all custom options created
        if (!$this->getIsActive()) {
            $customOptionsIdsToDelete = $this->getResource()->loadCustomOptionsToDelete($this->getId());
            $this->deleteCustomOptions($customOptionsIdsToDelete);
//            $this->getResource()->deleteAllIds($this->getId());
            return;
        }

//        $oldMatchingStoreIds = $this->getResource()->loadMatchingStoreIds($this->getId());
//        $newMatchingStoreIds = explode(',', $this->getData('store_ids'));
//        
//        $storeIdsToDelete = array_diff($oldMatchingStoreIds, $newMatchingStoreIds);
//        $this->getResource()->deleteUnmatchingStoreIds($storeIdsToDelete, $this->getId());
//        foreach ($newMatchingStoreIds as $storeId) {
//            $storeId = 0;
        $collection = $this->productCollectionFactory->create();
//        $collection->setVisibility($this->catalogProductVisibility->getVisibleInCatalogIds());
//            $collection->addStoreFilter($storeId);

        $conditions = $this->getConditions();
        $conditions->collectValidatedAttributes($collection);
        $this->sqlBuilder->attachConditionToCollection($collection, $conditions);
        $collection->distinct(true);

        $oldMatchingProductIds = $this->getResource()->loadMatchingProducts($this->getId());
//            $newMatchingProductIds = $collection->getAllIds();
        $newMatchingProductIds = [];

        foreach ($collection as $product) {
            $newMatchingProductIds[] = ($product->getRowId() ?: $product->getId());
        }

        $productIdsToDelete = array_diff($oldMatchingProductIds, $newMatchingProductIds);

        $customOptionsIdsToDeleteByProductIds = $this->getResource()->loadCustomOptionsToDeleteByProductIds($this->getId(), $productIdsToDelete);
        $this->deleteCustomOptions($customOptionsIdsToDeleteByProductIds);
//        $this->getResource()->deleteUnmatchingProductIds($productIdsToDelete, $this->getId());

        $productIdsToInsert = array_diff($newMatchingProductIds, $oldMatchingProductIds);

        $data = [];

        foreach ($productIdsToInsert as $productId) {
            foreach ($options as $arrayOption) {
                $data[] = [
                    'product_id' => $productId,
                    'dynamiccustomoptions_id' => $this->getId(),
                    'option_id' => $this->insertCustomOption($productId, $arrayOption)
                ];
            }
        }

        if (count($data)) {
            $this->getResource()->insertMatchingProducts($data);
        }


//        }
    }

    protected function deleteCustomOptions($customOptionsIdsToDelete) {
        foreach ($customOptionsIdsToDelete as $optionId) {
            $option = $this->productOptionFactory->create();
            $option->load($optionId);
            $option->delete();
        }
    }

    protected function insertCustomOption($productId, $arrayOption) {

        $option = $this->productOptionFactory->create()
                ->setProductId($productId)
                ->setStoreId(\Magento\Store\Model\Store::DEFAULT_STORE_ID)
                ->addData($arrayOption);
        $option->save();

        if (isset($arrayOption["translations"])) {
            
            $defaultValues = $option->getValuesCollection();
            
            foreach ($arrayOption["translations"] as $storeTranslations) {
                $option->setStoreId($storeTranslations["store_id"]);
                if (isset($storeTranslations["values"])) {
                    foreach ($storeTranslations["values"] as $storeValues) {
                        $option->setData($storeValues["attr_code"], $storeValues["attr_value"]);
                    }
                }
                if (isset($storeTranslations["type_values"])) {
                    $option->setData("values", $defaultValues->getData());
                    $newTranslatedData = [];
                    foreach ($defaultValues as $translateValue) {
                        foreach ($storeTranslations["type_values"] as $storeValues) {
                            if ($translateValue->getData($storeValues["primary_attr_code"]) == $storeValues["primary_original_value"]) {
                                $translateValue->setData($storeValues["primary_attr_code"], $storeValues["primary_attr_value"]);
                                if (isset($storeValues["other_attr"])) {
                                    foreach ($storeValues["other_attr"] as $otherValues) {
                                        $translateValue->setData($otherValues["attr_code"], $otherValues["attr_value"]);
                                    }
                                }
                            }
                        }
                        $newTranslatedData[] = $translateValue->getData();
                    }

                    $option->setData("values", $newTranslatedData);
                }
                else {
                    $option->setData("values", $defaultValues->getData());
//                    $newTranslatedData = [];
//                    foreach ($defaultValues as $translateValue) {
//                        $newTranslatedData[] = $translateValue->getData();
//                    }

//                    $option->setData("values", $newTranslatedData);
                }

                $option->save();
            }
        }

        // This is to avoid default price on the last inserted storeId
//        $option->setStoreId(\Magento\Store\Model\Store::DEFAULT_STORE_ID)
//                ->addData($arrayOption);
//        $option->save();

        return $option->getId();
    }

}
